------------------------------------------------------------
-- Talents.lua
--
-- Abin
-- 2011/11/13
------------------------------------------------------------

local InCombatLockdown = InCombatLockdown
local IsFlying = IsFlying
local GetActiveTalentGroup = GetActiveTalentGroup
local GetPrimaryTalentTree = GetPrimaryTalentTree
local GetTalentTabInfo = GetTalentTabInfo
local tostring = tostring
local GetNumTalentGroups = GetNumTalentGroups

local _, addon = ...
local L = addon.L

local QUESTIONMARK = "Interface\\Icons\\INV_Misc_QuestionMark"

local spellList = {}
spellList[1] = { icon = QUESTIONMARK, name = NONE, desc = "0/0/0" }
spellList[2] = { icon = QUESTIONMARK, name = NONE, desc = "0/0/0" }

local dualTalents

local button = addon:CreateActionButton("TalentSwitch", L["switch talents"])
button:SetFlyProtect(nil)

button:SetScript("OnClick", function(self)
	if not InCombatLockdown() and not IsFlying() then
		SetActiveTalentGroup(GetActiveTalentGroup() == 1 and 2 or 1)
	end
end)

function button:OnValidate()
	return GetNumTalentGroups() > 1
end

local function ValidateButton()
	if button:IsEnabled() and button:OnValidate() then
		button:Show()
	else
		button:Hide()
	end
end

local function UpdateSpecInfo(id)
	local spec = GetPrimaryTalentTree(false, false, id)
	local i, desc
	local maxName = NONE
	local maxIcon = QUESTIONMARK
	local maxPoints = 0
	for i = 1, 3 do
		local _, name, _, icon, points = GetTalentTabInfo(i, false, false, id)
		if desc then
			desc = desc.."/"..points
		else
			desc = tostring(points)
		end

		if points > maxPoints then
			maxName, maxIcon, maxPoints = name, icon, points
		end
	end

	spellList[id].name, spellList[id].icon, spellList[id].desc = maxName, maxIcon, desc
end

function button:OnTalentUpdate(combat)
	if not combat then
		ValidateButton()
	end

	if self:OnValidate() then
		UpdateSpecInfo(1)
		UpdateSpecInfo(2)
		self:UpdateTimer()
	end
end

function button:OnTooltipText(tooltip)
	if GetActiveTalentGroup() == 2 then
		tooltip:AddLine(spellList[1].name..": "..spellList[1].desc, 0.6, 0.6, 0.6, 1)
		tooltip:AddLine(spellList[2].name..": "..spellList[2].desc, 0, 1, 0, 1)
	else
		tooltip:AddLine(spellList[1].name..": "..spellList[1].desc, 0, 1, 0, 1)
		tooltip:AddLine(spellList[2].name..": "..spellList[2].desc, 0.6, 0.6, 0.6, 1)
	end
end

function button:OnUpdateTimer()
	local spec = GetActiveTalentGroup()
	self.icon:SetSpell(spellList[1])
	self.icon2:SetSpell(spellList[2])
	self.text:SetText(spellList[spec].desc)
	if spec == 2 then
		self.icon:SetAlpha(0.4)
		self.icon2:SetAlpha(1)
	else
		self.icon:SetAlpha(1)
		self.icon2:SetAlpha(0.4)
	end
	return 1
end

function button:OnLeaveCombat()
	ValidateButton()
end